@@ -8,7 +8,7 @@
 r"""
 .. dialect:: sqlite
     :name: SQLite
-    :full_support: 3.21, 3.28+
+    :full_support: 3.36.0
     :normal_support: 3.12+
     :best_effort: 3.7.16+
 
