@@ -1,5 +1,5 @@
 # orm/attributes.py
-# Copyright (C) 2005-2022 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2023 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -54,6 +54,8 @@ from ..sql import base as sql_base
 from ..sql import roles
 from ..sql import traversals
 from ..sql import visitors
+from ..sql.traversals import HasCacheKey
+from ..sql.visitors import InternalTraversal
 
 
 class NoKey(str):
@@ -223,13 +225,16 @@ class QueryableAttribute(
         subclass representing a column expression.
 
         """
+        entity_namespace = self._entity_namespace
+        assert isinstance(entity_namespace, HasCacheKey)
+
         if self.key is NO_KEY:
-            annotations = {"entity_namespace": self._entity_namespace}
+            annotations = {"entity_namespace": entity_namespace}
         else:
             annotations = {
                 "proxy_key": self.key,
                 "proxy_owner": self._parententity,
-                "entity_namespace": self._entity_namespace,
+                "entity_namespace": entity_namespace,
             }
 
         ce = self.comparator.__clause_element__()
@@ -376,7 +381,8 @@ else:
 
 
 class Mapped(QueryableAttribute, _Generic_T):
-    """Represent an ORM mapped :term:`descriptor` attribute for typing purposes.
+    """Represent an ORM mapped :term:`descriptor` attribute for typing
+    purposes.
 
     This class represents the complete descriptor interface for any class
     attribute that will have been :term:`instrumented` by the ORM
@@ -481,10 +487,22 @@ class InstrumentedAttribute(Mapped):
             return self.impl.get(state, dict_)
 
 
-HasEntityNamespace = util.namedtuple(
-    "HasEntityNamespace", ["entity_namespace"]
-)
-HasEntityNamespace.is_mapper = HasEntityNamespace.is_aliased_class = False
+class HasEntityNamespace(HasCacheKey):
+    __slots__ = ("_entity_namespace",)
+
+    is_mapper = False
+    is_aliased_class = False
+
+    _traverse_internals = [
+        ("_entity_namespace", InternalTraversal.dp_has_cache_key),
+    ]
+
+    def __init__(self, ent):
+        self._entity_namespace = ent
+
+    @property
+    def entity_namespace(self):
+        return self._entity_namespace.entity_namespace
 
 
 def create_proxied_attribute(descriptor):
@@ -549,7 +567,7 @@ def create_proxied_attribute(descriptor):
             else:
                 # used by hybrid attributes which try to remain
                 # agnostic of any ORM concepts like mappers
-                return HasEntityNamespace(self.class_)
+                return HasEntityNamespace(self._parententity)
 
         @property
         def property(self):
@@ -1584,7 +1602,13 @@ class CollectionAttributeImpl(AttributeImpl):
 
         self.dispatch.bulk_replace(state, new_values, evt)
 
-        old = self.get(state, dict_, passive=PASSIVE_ONLY_PERSISTENT)
+        # propagate NO_RAISE in passive through to the get() for the
+        # existing object (ticket #8862)
+        old = self.get(
+            state,
+            dict_,
+            passive=PASSIVE_ONLY_PERSISTENT ^ (passive & NO_RAISE),
+        )
         if old is PASSIVE_NO_RESULT:
             old = self._default_value(state, dict_)
         elif old is orig_iterable:
