@@ -1,5 +1,5 @@
 # sql/ddl.py
-# Copyright (C) 2009-2022 the SQLAlchemy authors and contributors
+# Copyright (C) 2009-2023 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -312,8 +312,10 @@ class DDL(DDLElement):
 
         :param statement:
           A string or unicode string to be executed.  Statements will be
-          processed with Python's string formatting operator.  See the
-          ``context`` argument and the ``execute_at`` method.
+          processed with Python's string formatting operator using
+          a fixed set of string substitutions, as well as additional
+          substitutions provided by the optional :paramref:`.DDL.context`
+          parameter.
 
           A literal '%' in a statement must be escaped as '%%'.
 
@@ -420,6 +422,8 @@ class CreateSchema(_CreateDropBase):
 
     __visit_name__ = "create_schema"
 
+    stringify_dialect = "default"
+
     def __init__(self, name, quote=None, **kw):
         """Create a new :class:`.CreateSchema` construct."""
 
@@ -436,6 +440,8 @@ class DropSchema(_CreateDropBase):
 
     __visit_name__ = "drop_schema"
 
+    stringify_dialect = "default"
+
     def __init__(self, name, quote=None, cascade=False, **kw):
         """Create a new :class:`.DropSchema` construct."""
 
