@@ -1,5 +1,5 @@
 # sql/lambdas.py
-# Copyright (C) 2005-2022 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2023 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -9,6 +9,7 @@ import inspect
 import itertools
 import operator
 import sys
+import threading
 import types
 import weakref
 
@@ -17,7 +18,6 @@ from . import elements
 from . import roles
 from . import schema
 from . import traversals
-from . import type_api
 from . import visitors
 from .base import _clone
 from .base import Options
@@ -218,11 +218,17 @@ class LambdaElement(elements.ClauseElement):
 
         if rec is None:
             if cache_key is not traversals.NO_CACHE:
-                rec = AnalyzedFunction(
-                    tracker, self, apply_propagate_attrs, fn
-                )
-                rec.closure_bindparams = bindparams
-                lambda_cache[tracker_key + cache_key] = rec
+
+                with AnalyzedCode._generation_mutex:
+                    key = tracker_key + cache_key
+                    if key not in lambda_cache:
+                        rec = AnalyzedFunction(
+                            tracker, self, apply_propagate_attrs, fn
+                        )
+                        rec.closure_bindparams = bindparams
+                        lambda_cache[key] = rec
+                    else:
+                        rec = lambda_cache[key]
             else:
                 rec = NonAnalyzedFunction(self._invoke_user_fn(fn))
 
@@ -607,6 +613,8 @@ class AnalyzedCode(object):
     )
     _fns = weakref.WeakKeyDictionary()
 
+    _generation_mutex = threading.RLock()
+
     @classmethod
     def get(cls, fn, lambda_element, lambda_kw, **kw):
         try:
@@ -614,10 +622,16 @@ class AnalyzedCode(object):
             return cls._fns[fn.__code__]
         except KeyError:
             pass
-        cls._fns[fn.__code__] = analyzed = AnalyzedCode(
-            fn, lambda_element, lambda_kw, **kw
-        )
-        return analyzed
+
+        with cls._generation_mutex:
+            # check for other thread already created object
+            if fn.__code__ in cls._fns:
+                return cls._fns[fn.__code__]
+
+            cls._fns[fn.__code__] = analyzed = AnalyzedCode(
+                fn, lambda_element, lambda_kw, **kw
+            )
+            return analyzed
 
     def __init__(self, fn, lambda_element, opts):
         if inspect.ismethod(fn):
@@ -1200,11 +1214,11 @@ class PyWrapper(ColumnOperators):
             return value
 
     def operate(self, op, *other, **kwargs):
-        elem = object.__getattribute__(self, "__clause_element__")()
+        elem = object.__getattribute__(self, "_py_wrapper_literal")()
         return op(elem, *other, **kwargs)
 
     def reverse_operate(self, op, other, **kwargs):
-        elem = object.__getattribute__(self, "__clause_element__")()
+        elem = object.__getattribute__(self, "_py_wrapper_literal")()
         return op(other, elem, **kwargs)
 
     def _extract_bound_parameters(self, starting_point, result_list):
@@ -1217,16 +1231,19 @@ class PyWrapper(ColumnOperators):
             element = getter(starting_point)
             pywrapper._sa__extract_bound_parameters(element, result_list)
 
-    def __clause_element__(self):
+    def _py_wrapper_literal(self, expr=None, operator=None, **kw):
         param = object.__getattribute__(self, "_param")
         to_evaluate = object.__getattribute__(self, "_to_evaluate")
         if param is None:
             name = object.__getattribute__(self, "_name")
             self._param = param = elements.BindParameter(
-                name, required=False, unique=True
+                name,
+                required=False,
+                unique=True,
+                _compared_to_operator=operator,
+                _compared_to_type=expr.type if expr is not None else None,
             )
             self._has_param = True
-            param.type = type_api._resolve_value_to_type(to_evaluate)
         return param._with_value(to_evaluate, maintain_key=True)
 
     def __bool__(self):
@@ -1244,6 +1261,7 @@ class PyWrapper(ColumnOperators):
             "__clause_element__",
             "operate",
             "reverse_operate",
+            "_py_wrapper_literal",
             "__class__",
             "__dict__",
         ):
