@@ -1,5 +1,5 @@
 # sql/types_api.py
-# Copyright (C) 2005-2022 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2023 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -22,6 +22,7 @@ from .. import util
 BOOLEANTYPE = None
 INTEGERTYPE = None
 NULLTYPE = None
+NUMERICTYPE = None
 STRINGTYPE = None
 MATCHTYPE = None
 INDEXABLE = None
@@ -110,8 +111,12 @@ class TypeEngine(Traversible):
 
             return op, self.type
 
+        # note: this reduce is needed for tests to pass under python 2.
+        # it does not appear to apply to python 3.  It has however been
+        # modified to accommodate issue #10213.  In SQLA 2 this reduce
+        # has been removed.
         def __reduce__(self):
-            return _reconstitute_comparator, (self.expr,)
+            return _reconstitute_comparator, (self.expr, self.expr.type)
 
     hashable = True
     """Flag, if False, means values from this type aren't hashable.
@@ -178,8 +183,8 @@ class TypeEngine(Traversible):
     """
 
     def evaluates_none(self):
-        """Return a copy of this type which has the :attr:`.should_evaluate_none`
-        flag set to True.
+        """Return a copy of this type which has the
+        :attr:`.should_evaluate_none` flag set to True.
 
         E.g.::
 
@@ -542,6 +547,11 @@ class TypeEngine(Traversible):
         :param dialect_name: base name of the dialect which uses
          this type. (i.e. ``'postgresql'``, ``'mysql'``, etc.)
 
+        .. seealso::
+
+            :ref:`types_with_variant` - illustrates the use of
+            :meth:`_types.TypeEngine.with_variant`.
+
         """
         return Variant(self, {dialect_name: to_instance(type_)})
 
@@ -744,7 +754,9 @@ class TypeEngine(Traversible):
                 else self.__dict__[k],
             )
             for k in names
-            if k in self.__dict__ and not k.startswith("_")
+            if k in self.__dict__
+            and not k.startswith("_")
+            and self.__dict__[k] is not None
         )
 
     def adapt(self, cls, **kw):
@@ -1104,7 +1116,8 @@ class Emulated(object):
     """
 
     def adapt_to_emulated(self, impltype, **kw):
-        """Given an impl class, adapt this type to the impl assuming "emulated".
+        """Given an impl class, adapt this type to the impl assuming
+        "emulated".
 
         The impl should also be an "emulated" version of this type,
         most likely the same class as this type itself.
@@ -1151,7 +1164,8 @@ class NativeForEmulated(object):
 
     @classmethod
     def adapt_emulated_to_native(cls, impl, **kw):
-        """Given an impl, adapt this type's class to the impl assuming "native".
+        """Given an impl, adapt this type's class to the impl assuming
+        "native".
 
         The impl will be an :class:`.Emulated` class but not a
         :class:`.NativeForEmulated`.
@@ -1264,8 +1278,11 @@ class TypeDecorator(ExternalType, SchemaEventTarget, TypeEngine):
        the default rules of :meth:`.TypeEngine.coerce_compared_value` should
        be used in order to deal with operators like index operations::
 
+            from sqlalchemy import JSON
+            from sqlalchemy import TypeDecorator
+
             class MyJsonType(TypeDecorator):
-                impl = postgresql.JSON
+                impl = JSON
 
                 cache_ok = True
 
@@ -1275,6 +1292,24 @@ class TypeDecorator(ExternalType, SchemaEventTarget, TypeEngine):
        Without the above step, index operations such as ``mycol['foo']``
        will cause the index value ``'foo'`` to be JSON encoded.
 
+       Similarly, when working with the :class:`.ARRAY` datatype, the
+       type coercion for index operations (e.g. ``mycol[5]``) is also
+       handled by :meth:`.TypeDecorator.coerce_compared_value`, where
+       again a simple override is sufficient unless special rules are needed
+       for particular operators::
+
+            from sqlalchemy import ARRAY
+            from sqlalchemy import TypeDecorator
+
+            class MyArrayType(TypeDecorator):
+                impl = ARRAY
+
+                cache_ok = True
+
+                def coerce_compared_value(self, op, value):
+                    return self.impl.coerce_compared_value(op, value)
+
+
     """
 
     __visit_name__ = "type_decorator"
@@ -1914,8 +1949,14 @@ class Variant(TypeDecorator):
         return self.impl.comparator_factory
 
 
-def _reconstitute_comparator(expression):
-    return expression.comparator
+def _reconstitute_comparator(expression, type_=None):
+    # changed for #10213, added type_ argument.
+    # for previous pickles, keep type_ optional
+    if type_ is None:
+        return expression.comparator
+
+    comparator_factory = type_.comparator_factory
+    return comparator_factory(expression)
 
 
 def to_instance(typeobj, *arg, **kw):
