@@ -9,10 +9,12 @@ import sys
 from sqlalchemy import testing
 from sqlalchemy.testing import config
 from sqlalchemy.testing import fixtures
+from sqlalchemy.testing import requires
 
 
 class DocTest(fixtures.TestBase):
     __requires__ = ("python3",)
+    __only_on__ = "sqlite+pysqlite"
 
     def _setup_logger(self):
         rootlogger = logging.getLogger("sqlalchemy.engine.Engine")
@@ -88,6 +90,7 @@ class DocTest(fixtures.TestBase):
                 globs.update(test.globs)
                 assert not runner.failures
 
+    @requires.has_json_each
     def test_20_style(self):
         self._run_doctest(
             "tutorial/index.rst",
@@ -115,6 +118,9 @@ class DocTest(fixtures.TestBase):
     def test_orm_queryguide(self):
         self._run_doctest("orm/queryguide.rst")
 
+    def test_orm_quickstart(self):
+        self._run_doctest("orm/quickstart.rst")
+
 
 # unicode checker courtesy pytest
 
