@@ -750,8 +750,8 @@ class PoolEventsTest(PoolTestBase):
         assert canary.call_args_list[0][0][0] is dbapi_con
         assert canary.call_args_list[0][0][2] is exc
 
-    @testing.combinations((True,), (False,), argnames="is_asyncio")
-    @testing.combinations((True,), (False,), argnames="has_terminate")
+    @testing.variation("is_asyncio", [(True, testing.requires.asyncio), False])
+    @testing.variation("has_terminate", [True, False])
     @testing.requires.python3
     def test_checkin_event_gc(self, is_asyncio, has_terminate):
         p, canary = self._checkin_event_fixture(
@@ -1684,7 +1684,12 @@ class QueuePoolTest(PoolTestBase):
             exc_cls=TimeoutThing if exc_type.base_exception else Exception,
         )
 
-    @testing.combinations((True, testing.requires.python3), (False,))
+    @testing.variation(
+        "detach_gced",
+        [("detached_gc", testing.requires.asyncio), "normal_gc"],
+    )
+    @testing.emits_warning("The garbage collector")
+    @testing.requires.python3
     def test_userspace_disconnectionerror_weakref_finalizer(self, detach_gced):
         dbapi, pool = self._queuepool_dbapi_fixture(
             pool_size=1, max_overflow=2, _is_asyncio=detach_gced
