@@ -2,10 +2,14 @@ from sqlalchemy import exc
 from sqlalchemy import ForeignKey
 from sqlalchemy import inspect
 from sqlalchemy import Integer
+from sqlalchemy import literal
 from sqlalchemy import select
 from sqlalchemy import String
 from sqlalchemy import testing
+from sqlalchemy import union
 from sqlalchemy.orm import backref
+from sqlalchemy.orm import column_property
+from sqlalchemy.orm import composite
 from sqlalchemy.orm import defaultload
 from sqlalchemy.orm import immediateload
 from sqlalchemy.orm import joinedload
@@ -21,11 +25,13 @@ from sqlalchemy.sql.selectable import LABEL_STYLE_TABLENAME_PLUS_COL
 from sqlalchemy.testing import assertsql
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import fixtures
+from sqlalchemy.testing import mock
 from sqlalchemy.testing.assertions import expect_raises_message
 from sqlalchemy.testing.assertsql import AllOf
 from sqlalchemy.testing.assertsql import CompiledSQL
 from sqlalchemy.testing.assertsql import EachOf
 from sqlalchemy.testing.assertsql import Or
+from sqlalchemy.testing.entities import ComparableEntity
 from sqlalchemy.testing.fixtures import fixture_session
 from sqlalchemy.testing.schema import Column
 from ._poly_fixtures import _Polymorphic
@@ -423,7 +429,10 @@ class TestGeometries(GeometryFixtureBase):
         with self.assert_statement_count(testing.db, 0):
             eq_(result, [d(d_data="d1"), e(e_data="e1")])
 
-    def test_threelevel_selectin_to_inline_awkward_alias_options(self):
+    @testing.combinations((True,), (False,))
+    def test_threelevel_selectin_to_inline_awkward_alias_options(
+        self, use_aliased_class
+    ):
         self._fixture_from_geometry(
             {
                 "a": {
@@ -454,57 +463,99 @@ class TestGeometries(GeometryFixtureBase):
         )
 
         c_alias = with_polymorphic(c, (d, e), poly)
-        q = (
-            sess.query(a)
-            .options(selectin_polymorphic(a, [b, c_alias]))
-            .order_by(a.id)
-        )
 
-        result = self.assert_sql_execution(
-            testing.db,
-            q.all,
-            CompiledSQL(
-                "SELECT a.type AS a_type, a.id AS a_id, "
-                "a.a_data AS a_a_data FROM a ORDER BY a.id",
-                {},
-            ),
-            Or(
-                # here, the test is that the adaptation of "a" takes place
+        if use_aliased_class:
+            opt = selectin_polymorphic(a, [b, c_alias])
+        else:
+            opt = selectin_polymorphic(
+                a,
+                [b, c_alias, d, e],
+            )
+        q = sess.query(a).options(opt).order_by(a.id)
+
+        if use_aliased_class:
+            result = self.assert_sql_execution(
+                testing.db,
+                q.all,
                 CompiledSQL(
-                    "SELECT poly.a_type AS poly_a_type, "
-                    "poly.c_id AS poly_c_id, "
-                    "poly.a_id AS poly_a_id, poly.c_c_data AS poly_c_c_data, "
-                    "poly.e_id AS poly_e_id, poly.e_e_data AS poly_e_e_data, "
-                    "poly.d_id AS poly_d_id, poly.d_d_data AS poly_d_d_data "
-                    "FROM (SELECT a.id AS a_id, a.type AS a_type, "
-                    "c.id AS c_id, "
-                    "c.c_data AS c_c_data, d.id AS d_id, "
-                    "d.d_data AS d_d_data, "
-                    "e.id AS e_id, e.e_data AS e_e_data FROM a JOIN c "
-                    "ON a.id = c.id LEFT OUTER JOIN d ON c.id = d.id "
-                    "LEFT OUTER JOIN e ON c.id = e.id) AS poly "
-                    "WHERE poly.a_id IN (__[POSTCOMPILE_primary_keys]) "
-                    "ORDER BY poly.a_id",
-                    [{"primary_keys": [1, 2]}],
+                    "SELECT a.type AS a_type, a.id AS a_id, "
+                    "a.a_data AS a_a_data FROM a ORDER BY a.id",
+                    {},
                 ),
+                Or(
+                    # here, the test is that the adaptation of "a" takes place
+                    CompiledSQL(
+                        "SELECT poly.a_type AS poly_a_type, "
+                        "poly.c_id AS poly_c_id, "
+                        "poly.a_id AS poly_a_id, poly.c_c_data "
+                        "AS poly_c_c_data, "
+                        "poly.e_id AS poly_e_id, poly.e_e_data "
+                        "AS poly_e_e_data, "
+                        "poly.d_id AS poly_d_id, poly.d_d_data "
+                        "AS poly_d_d_data "
+                        "FROM (SELECT a.id AS a_id, a.type AS a_type, "
+                        "c.id AS c_id, "
+                        "c.c_data AS c_c_data, d.id AS d_id, "
+                        "d.d_data AS d_d_data, "
+                        "e.id AS e_id, e.e_data AS e_e_data FROM a JOIN c "
+                        "ON a.id = c.id LEFT OUTER JOIN d ON c.id = d.id "
+                        "LEFT OUTER JOIN e ON c.id = e.id) AS poly "
+                        "WHERE poly.a_id IN (__[POSTCOMPILE_primary_keys]) "
+                        "ORDER BY poly.a_id",
+                        [{"primary_keys": [1, 2]}],
+                    ),
+                    CompiledSQL(
+                        "SELECT poly.a_type AS poly_a_type, "
+                        "poly.c_id AS poly_c_id, "
+                        "poly.a_id AS poly_a_id, "
+                        "poly.c_c_data AS poly_c_c_data, "
+                        "poly.d_id AS poly_d_id, poly.d_d_data "
+                        "AS poly_d_d_data, "
+                        "poly.e_id AS poly_e_id, poly.e_e_data "
+                        "AS poly_e_e_data "
+                        "FROM (SELECT a.id AS a_id, a.type AS a_type, "
+                        "c.id AS c_id, c.c_data AS c_c_data, d.id AS d_id, "
+                        "d.d_data AS d_d_data, e.id AS e_id, "
+                        "e.e_data AS e_e_data FROM a JOIN c ON a.id = c.id "
+                        "LEFT OUTER JOIN d ON c.id = d.id "
+                        "LEFT OUTER JOIN e ON c.id = e.id) AS poly "
+                        "WHERE poly.a_id IN (__[POSTCOMPILE_primary_keys]) "
+                        "ORDER BY poly.a_id",
+                        [{"primary_keys": [1, 2]}],
+                    ),
+                ),
+            )
+        else:
+            result = self.assert_sql_execution(
+                testing.db,
+                q.all,
                 CompiledSQL(
-                    "SELECT poly.a_type AS poly_a_type, "
-                    "poly.c_id AS poly_c_id, "
-                    "poly.a_id AS poly_a_id, poly.c_c_data AS poly_c_c_data, "
-                    "poly.d_id AS poly_d_id, poly.d_d_data AS poly_d_d_data, "
-                    "poly.e_id AS poly_e_id, poly.e_e_data AS poly_e_e_data "
-                    "FROM (SELECT a.id AS a_id, a.type AS a_type, "
-                    "c.id AS c_id, c.c_data AS c_c_data, d.id AS d_id, "
-                    "d.d_data AS d_d_data, e.id AS e_id, "
-                    "e.e_data AS e_e_data FROM a JOIN c ON a.id = c.id "
-                    "LEFT OUTER JOIN d ON c.id = d.id "
-                    "LEFT OUTER JOIN e ON c.id = e.id) AS poly "
-                    "WHERE poly.a_id IN (__[POSTCOMPILE_primary_keys]) "
-                    "ORDER BY poly.a_id",
-                    [{"primary_keys": [1, 2]}],
+                    "SELECT a.type AS a_type, a.id AS a_id, "
+                    "a.a_data AS a_a_data FROM a ORDER BY a.id",
+                    {},
                 ),
-            ),
-        )
+                AllOf(
+                    CompiledSQL(
+                        "SELECT a.type AS a_type, d.id AS d_id, c.id AS c_id, "
+                        "a.id AS a_id, "
+                        "d.d_data AS d_d_data FROM a "
+                        "JOIN c ON a.id = c.id JOIN d ON c.id = d.id "
+                        "WHERE a.id IN (__[POSTCOMPILE_primary_keys]) "
+                        "ORDER BY a.id",
+                        [{"primary_keys": [1]}],
+                    ),
+                    CompiledSQL(
+                        "SELECT a.type AS a_type, e.id AS e_id, c.id AS c_id, "
+                        "a.id AS a_id, "
+                        "e.e_data AS e_e_data FROM a "
+                        "JOIN c ON a.id = c.id JOIN e ON c.id = e.id "
+                        "WHERE a.id IN (__[POSTCOMPILE_primary_keys]) "
+                        "ORDER BY a.id",
+                        [{"primary_keys": [2]}],
+                    ),
+                ),
+            )
+
         with self.assert_statement_count(testing.db, 0):
             eq_(result, [d(d_data="d1"), e(e_data="e1")])
 
@@ -914,12 +965,327 @@ class LazyLoaderTransfersOptsTest(fixtures.DeclarativeMappedTest):
             _cache_key_traversal = ()
             propagate_to_loaders = True
 
-        any_opt = AnyOpt()
-        if strat is None:
-            opts = (any_opt,)
-        else:
-            opts = (strat(User.address), any_opt)
+            def _adjust_for_extra_criteria(self, context):
+                return self
 
-        u = sess.execute(select(User).options(*opts)).scalars().one()
-        address = u.address
-        eq_(inspect(address).load_options, set(opts))
+        from sqlalchemy.orm.strategy_options import Load
+
+        with mock.patch.object(
+            Load, "_adjust_for_extra_criteria", lambda self, ctx: self
+        ):
+            any_opt = AnyOpt()
+            if strat is None:
+                opts = (any_opt,)
+            else:
+                opts = (strat(User.address), any_opt)
+
+            u = sess.execute(select(User).options(*opts)).scalars().one()
+            address = u.address
+            eq_(inspect(address).load_options, opts)
+
+
+class NoBaseWPPlusAliasedTest(
+    testing.AssertsExecutionResults, fixtures.TestBase
+):
+    """test for #7799"""
+
+    @testing.fixture
+    def mapping_fixture(self, registry, connection):
+        Base = registry.generate_base()
+
+        class BaseClass(Base):
+            __tablename__ = "baseclass"
+            id = Column(
+                Integer,
+                primary_key=True,
+                unique=True,
+            )
+
+        class A(BaseClass):
+            __tablename__ = "a"
+
+            id = Column(ForeignKey(BaseClass.id), primary_key=True)
+            thing1 = Column(String(50))
+
+            __mapper_args__ = {"polymorphic_identity": "a"}
+
+        class B(BaseClass):
+            __tablename__ = "b"
+
+            id = Column(ForeignKey(BaseClass.id), primary_key=True)
+            thing2 = Column(String(50))
+
+            __mapper_args__ = {"polymorphic_identity": "b"}
+
+        registry.metadata.create_all(connection)
+        with Session(connection) as sess:
+
+            sess.add_all(
+                [
+                    A(thing1="thing1_1"),
+                    A(thing1="thing1_2"),
+                    B(thing2="thing2_2"),
+                    B(thing2="thing2_3"),
+                    A(thing1="thing1_3"),
+                    A(thing1="thing1_4"),
+                    B(thing2="thing2_1"),
+                    B(thing2="thing2_4"),
+                ]
+            )
+
+            sess.commit()
+
+        return BaseClass, A, B
+
+    def test_wp(self, mapping_fixture, connection):
+        BaseClass, A, B = mapping_fixture
+
+        stmt = union(
+            select(A.id, literal("a").label("type")),
+            select(B.id, literal("b").label("type")),
+        ).subquery()
+
+        wp = with_polymorphic(
+            BaseClass,
+            [A, B],
+            selectable=stmt,
+            polymorphic_on=stmt.c.type,
+        )
+
+        session = Session(connection)
+
+        with self.sql_execution_asserter() as asserter:
+            result = session.scalars(
+                select(wp)
+                .options(selectin_polymorphic(wp, [A, B]))
+                .order_by(wp.id)
+            )
+            for obj in result:
+                if isinstance(obj, A):
+                    obj.thing1
+                else:
+                    obj.thing2
+
+        asserter.assert_(
+            CompiledSQL(
+                "SELECT anon_1.id, anon_1.type FROM "
+                "(SELECT a.id AS id, :param_1 AS type FROM baseclass "
+                "JOIN a ON baseclass.id = a.id "
+                "UNION SELECT b.id AS id, :param_2 AS type "
+                "FROM baseclass JOIN b ON baseclass.id = b.id) AS anon_1 "
+                "ORDER BY anon_1.id",
+                [{"param_1": "a", "param_2": "b"}],
+            ),
+            AllOf(
+                CompiledSQL(
+                    "SELECT a.id AS a_id, baseclass.id AS baseclass_id, "
+                    "a.thing1 AS a_thing1 FROM baseclass "
+                    "JOIN a ON baseclass.id = a.id "
+                    "WHERE baseclass.id IN (__[POSTCOMPILE_primary_keys]) "
+                    "ORDER BY baseclass.id",
+                    {"primary_keys": [1, 2, 5, 6]},
+                ),
+                CompiledSQL(
+                    "SELECT b.id AS b_id, baseclass.id AS baseclass_id, "
+                    "b.thing2 AS b_thing2 FROM baseclass "
+                    "JOIN b ON baseclass.id = b.id "
+                    "WHERE baseclass.id IN (__[POSTCOMPILE_primary_keys]) "
+                    "ORDER BY baseclass.id",
+                    {"primary_keys": [3, 4, 7, 8]},
+                ),
+            ),
+        )
+
+
+class CompositeAttributesTest(fixtures.TestBase):
+    @testing.fixture
+    def mapping_fixture(self, registry, connection):
+        Base = registry.generate_base()
+
+        class BaseCls(Base):
+            __tablename__ = "base"
+            id = Column(
+                Integer, primary_key=True, test_needs_autoincrement=True
+            )
+            type = Column(String(50))
+
+            __mapper_args__ = {"polymorphic_on": type}
+
+        class XYThing:
+            def __init__(self, x, y):
+                self.x = x
+                self.y = y
+
+            def __composite_values__(self):
+                return (self.x, self.y)
+
+            def __eq__(self, other):
+                return (
+                    isinstance(other, XYThing)
+                    and other.x == self.x
+                    and other.y == self.y
+                )
+
+            def __ne__(self, other):
+                return not self.__eq__(other)
+
+        class A(ComparableEntity, BaseCls):
+            __tablename__ = "a"
+            id = Column(ForeignKey(BaseCls.id), primary_key=True)
+            thing1 = Column(String(50))
+            comp1 = composite(
+                XYThing, Column("x1", Integer), Column("y1", Integer)
+            )
+
+            __mapper_args__ = {
+                "polymorphic_identity": "a",
+                "polymorphic_load": "selectin",
+            }
+
+        class B(ComparableEntity, BaseCls):
+            __tablename__ = "b"
+            id = Column(ForeignKey(BaseCls.id), primary_key=True)
+            thing2 = Column(String(50))
+            comp2 = composite(
+                XYThing, Column("x2", Integer), Column("y2", Integer)
+            )
+
+            __mapper_args__ = {
+                "polymorphic_identity": "b",
+                "polymorphic_load": "selectin",
+            }
+
+        registry.metadata.create_all(connection)
+
+        with Session(connection) as sess:
+            sess.add_all(
+                [
+                    A(id=1, thing1="thing1", comp1=XYThing(1, 2)),
+                    B(id=2, thing2="thing2", comp2=XYThing(3, 4)),
+                ]
+            )
+            sess.commit()
+
+        return BaseCls, A, B, XYThing
+
+    def test_load_composite(self, mapping_fixture, connection):
+        BaseCls, A, B, XYThing = mapping_fixture
+
+        with Session(connection) as sess:
+            rows = sess.scalars(select(BaseCls).order_by(BaseCls.id)).all()
+
+            eq_(
+                rows,
+                [
+                    A(id=1, thing1="thing1", comp1=XYThing(1, 2)),
+                    B(id=2, thing2="thing2", comp2=XYThing(3, 4)),
+                ],
+            )
+
+
+class PolymorphicOnExprTest(
+    testing.AssertsExecutionResults, fixtures.TestBase
+):
+    """test for #8704"""
+
+    @testing.fixture()
+    def poly_fixture(self, connection, decl_base):
+        def fixture(create_prop, use_load):
+            class TypeTable(decl_base):
+                __tablename__ = "type"
+
+                id = Column(Integer, primary_key=True)
+                name = Column(String(30))
+
+            class PolyBase(ComparableEntity, decl_base):
+                __tablename__ = "base"
+
+                id = Column(Integer, primary_key=True)
+                type_id = Column(ForeignKey(TypeTable.id))
+
+                if create_prop == "create_prop":
+                    polymorphic = column_property(
+                        select(TypeTable.name)
+                        .where(TypeTable.id == type_id)
+                        .scalar_subquery()
+                    )
+                    __mapper_args__ = {
+                        "polymorphic_on": polymorphic,
+                    }
+                elif create_prop == "dont_create_prop":
+                    __mapper_args__ = {
+                        "polymorphic_on": select(TypeTable.name)
+                        .where(TypeTable.id == type_id)
+                        .scalar_subquery()
+                    }
+                elif create_prop == "arg_level_prop":
+                    __mapper_args__ = {
+                        "polymorphic_on": column_property(
+                            select(TypeTable.name)
+                            .where(TypeTable.id == type_id)
+                            .scalar_subquery()
+                        )
+                    }
+
+            class Foo(PolyBase):
+                __tablename__ = "foo"
+
+                if use_load == "use_polymorphic_load":
+                    __mapper_args__ = {
+                        "polymorphic_identity": "foo",
+                        "polymorphic_load": "selectin",
+                    }
+                else:
+                    __mapper_args__ = {
+                        "polymorphic_identity": "foo",
+                    }
+
+                id = Column(ForeignKey(PolyBase.id), primary_key=True)
+                foo_attr = Column(String(30))
+
+            decl_base.metadata.create_all(connection)
+
+            with Session(connection) as session:
+                foo_type = TypeTable(name="foo")
+                session.add(foo_type)
+                session.flush()
+
+                foo = Foo(type_id=foo_type.id, foo_attr="foo value")
+                session.add(foo)
+
+                session.commit()
+
+            return PolyBase, Foo, TypeTable
+
+        yield fixture
+
+    @testing.combinations(
+        "create_prop",
+        "dont_create_prop",
+        "arg_level_prop",
+        argnames="create_prop",
+    )
+    @testing.combinations(
+        "use_polymorphic_load",
+        "use_loader_option",
+        "none",
+        argnames="use_load",
+    )
+    def test_load_selectin(
+        self, poly_fixture, connection, create_prop, use_load
+    ):
+        PolyBase, Foo, TypeTable = poly_fixture(create_prop, use_load)
+
+        sess = Session(connection)
+
+        foo_type = sess.scalars(select(TypeTable)).one()
+
+        stmt = select(PolyBase)
+        if use_load == "use_loader_option":
+            stmt = stmt.options(selectin_polymorphic(PolyBase, [Foo]))
+        obj = sess.scalars(stmt).all()
+
+        def go():
+            eq_(obj, [Foo(type_id=foo_type.id, foo_attr="foo value")])
+
+        self.assert_sql_count(testing.db, go, 0 if use_load != "none" else 1)
