@@ -1360,6 +1360,10 @@ class DefaultRequirements(SuiteRequirements):
             ]
         )
 
+    @property
+    def literal_float_coercion(self):
+        return skip_if("+asyncmy")
+
     @property
     def infinity_floats(self):
         return fails_on_everything_except(
